﻿local SN, L, F, T, M, _ = unpack(select(2,...))
local LSM = LibStub("LibSharedMedia-3.0")
local MODNAME = "raidbuffs"
local RB = SN:NewModule(MODNAME, "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0")

local raidMembers = {}
local UpdateDelay = 0
local raidbuffs
local missingFood = {}
local missingFlask = {}

local function SpellName(spellID)
	local name = GetSpellInfo(spellID)
	return name
end

local function SpellIcon(spellID)
	local _, _, icon = GetSpellInfo(spellID)
	return icon
end

local player = {}
player.__index = player
function player:new()
	self = {}
	setmetatable(self, player)
	self.name = ""
	self.zone = ""
	self.flask = false
	self.food = false
	self.fortitude = false
	self.statbuff = false
	self.masterybuff = false
	self.spbuff = false
	self.critbuff = false
	
	return self
end

local buffs = {
	flask = 0,
	food = 0,
	fortitude = 0,
	statbuff = 0,
	masterybuff = 0,
	spbuff = 0,
	critbuff = 0,
}
local missingBuffs = {
	flask = {},
	food = {},
	fortitude = {},
	statbuff = {},
	masterybuff = {},
	spbuff = {},
	critbuff = {},
}

local bufftext = {
	["flask"] =       "{triangle}Flasks or Elixirs{triangle}",
	["food"] =        "{moon}Well Fed Buff{moon}",
	["fortitude"] =   "{square}Stamina Buff{square}",
	["statbuff"] =    "{diamond}Stat Buff{diamond}",
	["masterybuff"] = "{star}Mastery Buff{star}",
	["spbuff"] =      "{circle}Spell Power Buff{circle}",
	["critbuff"] =    "{skull}Crit Buff{skull}",
}

local buffspells = {
	["flask"] = {
		SpellName(105689),  -- Flask of Spring Blossoms
		SpellName(105691),  -- Flask of the Warm Sun
		SpellName(105693),  -- Flask of Falling Leaves
		SpellName(105694),  -- Flask of the Earth
		SpellName(105696),  -- Flask of Winter\'s Bite
		SpellName(105682),	-- Mad Hozen Elixir
		SpellName(105683),	-- Elixir of Weaponry
		SpellName(105684),	-- Elixir of the Rapids
		SpellName(105685),	-- Elixir of Peace
		SpellName(105686),	-- Elixir of Perfection
		SpellName(105688),	-- Monk\'s Elixir
		SpellName(105681),	-- Mantid Elixir
		SpellName(105687),	-- Elixir of Mirrors
		SpellName(127230),  -- Vision of Insanity
	}, 
	["food"] = {
		SpellName(35272),	-- Well Fed
	},
	["fortitude"] = {
		SpellName(21562), 	-- Prayer of Fortitude
		SpellName(109773), 	-- Dark Intent
		SpellName(469),     -- Commanding Shout
		SpellName(90364),   -- Qiraji Fortitude
		SpellName(111923),  -- Runescroll of Fortitude III
	},
	["statbuff"] = {
		SpellName(1126), 	-- Mark of the Wild
		SpellName(20217), 	-- Blessing of Kings
		SpellName(117666), 	-- Legacy of the Emperor
		SpellName(90363),   -- Embrace of the Shale Spider
	},
	["masterybuff"] = { 
		SpellName(19740),   -- Blessing of Might
		SpellName(116956),  -- Grace of Air
		SpellName(93435),   -- Roar of Courage
		SpellName(128997),  -- Spirit Beast Blessing        
	},
	["spbuff"] = { 
		SpellName(1459), 	-- Arcane Intellect
		SpellName(61316), 	-- Dalaran Brilliance
		SpellName(109773), 	-- Dark Intent
		SpellName(77747), 	-- Burning Wrath
		SpellName(126309), 	-- Still Water
	},
	["critbuff"] = {
		SpellName(116781),	-- Legacy of the White Tiger
		SpellName(17007), 	-- Leader of the Pack
		SpellName(1459), 	-- Arcane Brilliance
		SpellName(61316),	-- Dalaran Brilliance
		SpellName(24604), 	-- Furious Howl
		SpellName(90309), 	-- Terrifying Roar
		SpellName(97229), 	-- Bellowing Roar
		SpellName(126373),	-- Fearless Roar
		SpellName(126309),	-- Still Water        
	},
}

--local buffClasses = {
--	["fortitude"] = {
--		"PRIEST",
--		"WARLOCK",
--	},
--	["statbuff"] = {
--		"DRUID",
--		"PALADIN",
--		"MONK",
--	},
--	["masterybuff"] = {
--		"PALADIN",
--	},
--	["spbuff"] = {
--		"MAGE",
--		"WARLOCK",
--	},
--	["critbuff"] = {
--		"MAGE",
--	},
--}

local spellIcons = {
	flask = SpellIcon(105691),
	food = SpellIcon(35272),
	fortitude = SpellIcon(21562),
	statbuff = SpellIcon(20217),
	masterybuff = SpellIcon(19740),
	spbuff = SpellIcon(1459),
	critbuff = SpellIcon(17007),
}

local function getOptions()
	raidbuffs = {
		type = "group",
		name = L["Raid Buffs"],
		args = {
			enable = {
				type = "toggle",
				name = L["Enable Raid Buff System"],
				order = 0,
				get = function(info)
					return SN.data.profile.raidbuffs[info[#info]]
				end,
				set = function(info, value)
					SN.data.profile.raidbuffs[info[#info]] = value
					if value then
						SN:EnableModule(MODNAME)
					else
						SN:DisableModule(MODNAME)
					end
				end,
			},
			orientation = {
				order = 1,
				type = "select",
				name = L["Direction"],
				values = {
					["HORIZONTAL"] = "HORIZONTAL",
					["VERTICAL"] = "VERTICAL",
				},
				hidden = function() return not SN.data.profile.raidbuffs.enable end,
				get = function(info)
					return SN.data.profile.raidbuffs[info[#info]]
				end,
				set = function(info, value)
					SN.data.profile.raidbuffs[info[#info]] = value
					RB:ApplySettings()
				end,
			},
			whisper = {
				type = "toggle",
				name = L["Whisper people with missing Flask/Food"],
				order = 0,
				get = function(info)
					return SN.data.profile.raidbuffs[info[#info]]
				end,
				set = function(info, value)
					SN.data.profile.raidbuffs[info[#info]] = value
				end,
			},
			readycheck = {
				type = "toggle",
				name = L["Announce people with missing Flask/Food when a Ready Check is issued"],
				order = 0,
				get = function(info)
					return SN.data.profile.raidbuffs[info[#info]]
				end,
				set = function(info, value)
					SN.data.profile.raidbuffs[info[#info]] = value
				end,
			},
		},
	}
	return raidbuffs
end

function player:CheckBuffs(buff)
	--Check for buffs
	for k, v in pairs(buffspells[buff]) do
		if UnitBuff(self.name, buffspells[buff][k]) then
			self[buff] = true
			return
		end
	end
end

function RB:OnInitialize()
	SN:RegisterModuleOptions(MODNAME, getOptions, L["Raid Buffs"])

	--Raid Buff Window
	--
	RB.Buffs = SN:CreateFrame("Raid Buff Frame", MODNAME, UIParent)
	SN:CreateBackDrop(RB.Buffs)
	
	if SN.data.profile.raidbuffs.orientation == "VERTICAL" then
		RB.Buffs:SetSize(SN.data.profile.raidbuffs.height, SN.data.profile.raidbuffs.width)
	end
	
	local size = SN.data.profile.raidbuffs.height - 14
	local counter = ""
	for k, v in pairs(buffs) do		
		RB.Buffs[k] = CreateFrame("Button", nil, RB.Buffs)
		RB.Buffs[k]:SetSize(size, size)
		RB.Buffs[k]:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight")
		RB.Buffs[k]:SetNormalTexture(spellIcons[k])
		RB.Buffs[k]:SetParent(RB.Buffs)
		if counter == "" then
			if SN.data.profile.raidbuffs.orientation == "VERTICAL" then
				RB.Buffs[k]:SetPoint("TOPLEFT", RB.Buffs, 7, -7)
			else
				RB.Buffs[k]:SetPoint("LEFT", RB.Buffs, 7, 0)
			end
		else
			if SN.data.profile.raidbuffs.orientation == "VERTICAL" then
				RB.Buffs[k]:SetPoint("TOPLEFT", RB.Buffs[counter], "BOTTOMLEFT", 0, -7)
			else
				RB.Buffs[k]:SetPoint("LEFT", RB.Buffs[counter], "RIGHT", 7, 0)
			end
		end
		RB.Buffs[k]:SetScript("OnEnter", function(self, motion)
			GameTooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT")
			GameTooltip:SetText("Missing Buff:")
			if not missingBuffs[k] then
				GameTooltip:AddLine("none")
			else
				for a, b in pairs(missingBuffs[k]) do
					GameTooltip:AddLine(b)
				end
			end
			GameTooltip:Show()
		end)
		RB.Buffs[k]:SetScript("OnLeave", function()
			GameTooltip:ClearLines()
			GameTooltip:Hide()
		end)
		RB.Buffs[k]:SetScript("OnClick", function(self, button, down)
			RB:AnnounceBuffs()
		end)
		local color = NORMAL_FONT_COLOR
		RB.Buffs[k].text = RB.Buffs[k]:CreateFontString(nil, "ARTWORK")
		RB.Buffs[k].text:SetFontObject(GameFontGreenLarge )
		RB.Buffs[k].text:SetPoint("CENTER", RB.Buffs[k], "CENTER", 0, 0)
		RB.Buffs[k].text:SetText("0")
		RB.Buffs[k].text:Show()
		counter = k
		RB.Buffs[k].name = k
	end
	
	RB.Buffs:SetScript("OnEvent", function(self, event, ...)
	--	if SN.data.profile.debug.enable then
	--		SN:Print(event)
	--	end
		if event == "PLAYER_REGEN_ENABLED" or event == "PET_BATTLE_CLOSE" then
			if IsInGroup() and not InCombatLockdown() then
				RB:ToggleActive(true)
			end
		elseif event == "PLAYER_REGEN_DISABLED" or event == "PET_BATTLE_OPENING_START" then
			RB:ToggleActive(false)
		elseif event == "READY_CHECK" then
			RB:AnnounceBuffs()
		elseif event == "GROUP_ROSTER_UPDATE" then
			if GetNumGroupMembers() > 1 and not self:IsVisible() then
				RB:ToggleActive(true)
			end
			wipe(raidMembers)
			for i = 1, GetNumGroupMembers() do
				local name, _, _, _, _, _, inzone = GetRaidRosterInfo(i)
				if name then
					local cachename = name:gsub("^(.*)%-.*$", "%1")
					if not raidMembers[cachename] then
						raidMembers[cachename] = player:new()
						raidMembers[cachename].name = cachename
						raidMembers[cachename].zone = inzone
					end
				end
			end
		end
	end)
	RB.Buffs:Hide()
end

function RB:OnEnable()
	if not IsInGroup() then
		RB:ToggleActive(false)
		for k, v in pairs(buffs) do
			RB.Buffs[k].text:SetText("0")
			buffs[k] = 0
			if missingBuffs[k] then
				wipe(missingBuffs[k])
			end
		end
	else
		RB:ToggleActive(true)
		wipe(raidMembers)
		for i = 1, GetNumGroupMembers() do
			local name, _, _, _, _, _, inzone = GetRaidRosterInfo(i)
			if name then
				local cachename = name:gsub("^(.*)%-.*$", "%1")
				if not raidMembers[cachename] then
					raidMembers[cachename] = player:new()
					raidMembers[cachename].name = cachename
					raidMembers[cachename].zone = inzone
				end
			end
		end
		RB:RefreshBuffs()
		for k, v in pairs(buffs) do
			RB.Buffs[k].text:SetText(tostring(buffs[k]))
		end
		UpdateDelay = 0
	end
	RB.Buffs:RegisterEvent("PLAYER_REGEN_DISABLED")
	RB.Buffs:RegisterEvent("PLAYER_REGEN_ENABLED")
	RB.Buffs:RegisterEvent("GROUP_ROSTER_UPDATE")
	RB.Buffs:RegisterEvent("READY_CHECK")
	RB.Buffs:RegisterEvent("PET_BATTLE_OPENING_START")
	RB.Buffs:RegisterEvent("PET_BATTLE_CLOSE")
end

function RB:OnDisable()
	RB.Buffs:UnregisterAllEvents()
	RB:ToggleActive(false)
end

function RB:Lock()
	SN:LockFrame(RB.Buffs)
	for k, v in pairs(buffs) do		
		RB.Buffs[k]:Show()
	end
	SN:CreateBackDrop(RB.Buffs)
end

function RB:Unlock()
	local width
	local height

	for k, v in pairs(buffs) do		
		RB.Buffs[k]:Hide()
	end
	
	if SN.data.profile.raidbuffs.orientation == "VERTICAL" then
		width = RB.Buffs:GetWidth()
		height = RB.Buffs:GetHeight()

		SN.data.profile.raidbuffs.width = width
		SN.data.profile.raidbuffs.height = height

		SN:UnlockFrame(RB.Buffs)

		SN.data.profile.raidbuffs.width = height
		SN.data.profile.raidbuffs.height = width
	else
		SN:UnlockFrame(RB.Buffs)
	end
end

function RB:Reset()
	local static = SN.DB.profile.frames
	local temp = SN.data.profile.frames
	RB.Buffs:ClearAllPoints()

	RB.Buffs:SetPoint(unpack(static.raidbuffs))
	wipe(temp.raidbuffs)
end

function RB:ApplySettings()
	RB.Buffs:ClearAllPoints()
	RB.Buffs:SetPoint(unpack(SN.data.profile.frames.raidbuffs))

	if SN.data.profile.raidbuffs.orientation == "VERTICAL" then
		RB.Buffs:SetSize(SN.data.profile.raidbuffs.height, SN.data.profile.raidbuffs.width)
	else
		RB.Buffs:SetSize(SN.data.profile.raidbuffs.width, SN.data.profile.raidbuffs.height)
	end
	local counter = ""
	for k, v in pairs(buffs) do
		RB.Buffs[k]:ClearAllPoints()
		if counter == "" then
			if SN.data.profile.raidbuffs.orientation == "VERTICAL" then
				RB.Buffs[k]:SetPoint("TOPLEFT", RB.Buffs, 7, -7)
			else
				RB.Buffs[k]:SetPoint("LEFT", RB.Buffs, 7, 0)
			end
		else
			if SN.data.profile.raidbuffs.orientation == "VERTICAL" then
				RB.Buffs[k]:SetPoint("TOPLEFT", RB.Buffs[counter], "BOTTOMLEFT", 0, -7)
			else
				RB.Buffs[k]:SetPoint("LEFT", RB.Buffs[counter], "RIGHT", 7, 0)
			end
		end
		counter = k
	end
end

function RB:RefreshBuffs()
	local ZoneName = GetRealZoneText()

	for k, v in pairs(buffs) do
		buffs[k] = 0
	end
	for k, v in pairs(missingBuffs) do
		wipe(missingBuffs[k])
	end
	
	for k, v in pairs(raidMembers) do
		for i, j in pairs(raidMembers[k]) do
			if type(v) == "boolean" then
				raidMembers[k][i] = false
			end
		end
	end

	for k, v in pairs(raidMembers) do
		for i, j in pairs(buffs) do
			raidMembers[k]:CheckBuffs(i)
		end
	end

	for i, j in pairs(raidMembers) do
		for k, v in pairs(buffs) do
			if not raidMembers[i][k] then 
				buffs[k] = buffs[k] + 1
				missingBuffs[k][i] = i
			end
		end
	end
end

function RB:DelayToggle(args)
	RB:ToggleActive(unpack(args))
end

function RB:ToggleActive(bool)
	if bool then
		UpdateDelay = 0
		local handler = RB.Buffs:GetScript("OnUpdate")

		if not handler then
			RB.Buffs:SetScript("OnUpdate", function(self, elapsed)
				UpdateDelay = UpdateDelay + elapsed
				if UpdateDelay > 6 then
					if not IsInGroup() then
						for k, v in pairs(buffs) do
							self[k].text:SetText("0")
							buffs[k] = 0
							if missingBuffs[k] then
								wipe(missingBuffs[k])
							end
						end 
					else
						RB:RefreshBuffs()
						for k, v in pairs(buffs) do
							self[k].text:SetText(tostring(buffs[k]))
						end
					end
					UpdateDelay = 0
				end
			end)
		end
		RB.Buffs:Show()
	else
		RB.Buffs:SetScript("OnUpdate", nil)
		RB.Buffs:Hide()
		collectgarbage("collect")
	end
end

function RB:AnnounceBuffs()
	local report = {}
	local allgood = true
	local group
	
	if SN:GroupType() == 1 then
		group = "PARTY"
	elseif SN:GroupType() == 2 then
		group = "RAID"
	elseif SN:GroupType() == 3 then
		group = "INSTANCE_CHAT"
	end
			
	if group then
		SendChatMessage("{star}Serenity Now{star}", group)
		for a, b in pairs(buffs) do
			for c, d in pairs (missingBuffs[a]) do
				if not report[a] then
					report[a] = c
				else
					report[a] = strjoin(", ", report[a], c)
				end
				
				if a == "flask" then
					table.insert(missingFlask, c)
				elseif a == "food" then
					table.insert(missingFood, c)
				end
			end
			if report[a] then
				SendChatMessage("People missing "..bufftext[a]..": "..report[a], group)
			end
		end

		if SN.data.profile.raidbuffs.whisper then
			RB:ScheduleTimer("WhisperWarning", 5)
		end

		for a, b in pairs(report) do
			if b then
				allgood = false
				return
			end
		end

		if allgood then
			SendChatMessage("Buffs are all good", group)
		end
	end

end

function RB:WhisperWarning()
	--Msg the people missing flasks
	if #missingFlask > 0 then
		for k, v in pairs(missingFlask) do
			SendChatMessage("Missing "..bufftext["flask"], "WHISPER", nil, v)
		end
	end
	--Msg the people missing food
	if #missingFood > 0 then
		for k, v in pairs(missingFood) do
			SendChatMessage("Missing "..bufftext["food"], "WHISPER", nil, v)
		end
	end
	wipe(missingFlask)
	wipe(missingFood)
end