﻿------------------------------------------------------------------------------------------
--Warlock Resources-----------------------------------------------------------------------
------------------------------------------------------------------------------------------

local SN, L, F, T, M, _ = unpack(select(2,...))

if SN.playerClass ~= "WARLOCK" then return end

local LSM = LibStub("LibSharedMedia-3.0")
local CP = SN:GetModule("classResources")

local MODNAME = "shards"

local SHARD = CP:NewModule(MODNAME, "AceEvent-3.0")

local MAX_POWER_PER_EMBER = 10
local SPELL_POWER_DEMONIC_FURY = SPELL_POWER_DEMONIC_FURY
local SPELL_POWER_BURNING_EMBERS = SPELL_POWER_BURNING_EMBERS
local SPELL_POWER_SOUL_SHARDS = SPELL_POWER_SOUL_SHARDS
local SPEC_WARLOCK_DESTRUCTION = SPEC_WARLOCK_DESTRUCTION
local SPEC_WARLOCK_AFFLICTION = SPEC_WARLOCK_AFFLICTION
local SPEC_WARLOCK_DEMONOLOGY = SPEC_WARLOCK_DEMONOLOGY
local maxShard = SN["classMaxResourceBar"]["WARLOCK"]

function SHARD:OnInitialize()
	
	local frame = F.classpower
	
	frame.shards = CP:PowerBubbles(maxShard)

	frame.shards:SetScript("OnEvent", function(self, event, ...)
		SHARD:ShardEvents(self)
	end)
	frame.shards:Hide()
end

function SHARD:OnEnable()
	local frame = F.classpower
	frame.shards:Show()
	frame.shards:RegisterEvent("UNIT_POWER")
	frame.shards:RegisterEvent("PLAYER_TALENT_UPDATE")
	frame.shards:RegisterEvent("PLAYER_ENTERING_WORLD")
	frame.shards:RegisterEvent("UNIT_DISPLAYPOWER")
	SHARD:ShardEvents(frame.shards)
	WarlockPowerFrame:Hide()
	WarlockPowerFrame:UnregisterAllEvents()
	collectgarbage("collect")
end

function SHARD:OnDisable()
	
	local frame = F.classpower
	frame.shards:UnregisterAllEvents()
	frame.shards:Hide()
	WarlockPowerFrame:Show()
	WarlockPowerFrame:GetScript("OnLoad")(WarlockPowerFrame)
	collectgarbage("collect")
end

function SHARD:ShardEvents(self)
	local color = M.classResources.WARLOCK
	local spec = GetSpecialization()
	if spec then
		if not self:IsShown() then 
			self:Show()
		end

		for i = 1, maxShard do
			self[i]:Show()
			self[i]:SetStatusBarColor(color[spec][1], color[spec][2], color[spec][3], 1)
			self[i]:SetBackdropColor(color[spec][1], color[spec][2], color[spec][3], 0.2)
		end
		
		if (spec == SPEC_WARLOCK_DESTRUCTION) then	
			local maxPower = UnitPowerMax("player", SPELL_POWER_BURNING_EMBERS, true)
			local power = UnitPower("player", SPELL_POWER_BURNING_EMBERS, true)
			local numEmbers = power / MAX_POWER_PER_EMBER
			local numBars = floor(maxPower / MAX_POWER_PER_EMBER)
			self.number = numBars
			-- bar unavailable
			
			if numBars == 3 then
				self[4]:Hide()
			else
				self[4]:Show()
			end

			for i = 1, numBars do
				self[i]:SetSize((SN.data.profile.classResources.width - (numBars + 1) * 3) / numBars, SN.data.profile.classResources.height - 6)
				if i == 1 then
					self[i]:SetPoint("LEFT", self, 3, 0)
				else
					self[i]:SetPoint("LEFT", self[(i - 1)], "RIGHT", 3, 0)
				end

				self[i]:SetMinMaxValues((MAX_POWER_PER_EMBER * i) - MAX_POWER_PER_EMBER, MAX_POWER_PER_EMBER * i)
				self[i]:SetValue(power)
			end
		elseif ( spec == SPEC_WARLOCK_AFFLICTION ) then
			local numShards = UnitPower("player", SPELL_POWER_SOUL_SHARDS)
			local maxShards = UnitPowerMax("player", SPELL_POWER_SOUL_SHARDS)
			self.number = maxShards
			
			self[4]:Hide()

			for i = 1, maxShards do
				
				self[i]:SetSize((SN.data.profile.classResources.width - (maxShards + 1) * 3) / maxShards, SN.data.profile.classResources.height - 6)
				self[i]:SetMinMaxValues(0, 1)
				self[i]:SetValue(1)
				if i == 1 then
					self[i]:SetPoint("LEFT", self, 3, 0)
				else
					self[i]:SetPoint("LEFT", self[(i - 1)], "RIGHT", 3, 0)
				end
				if i <= numShards then
					self[i]:SetAlpha(1)
				else
					self[i]:SetAlpha(0.1)
				end
			end
		elseif spec == SPEC_WARLOCK_DEMONOLOGY then
			local power = UnitPower("player", SPELL_POWER_DEMONIC_FURY)
			local maxPower = UnitPowerMax("player", SPELL_POWER_DEMONIC_FURY)
			
			self.number = 1
			self[2]:Hide()
			self[3]:Hide()
			self[4]:Hide()

			self[1]:SetSize((SN.data.profile.classResources.width - 6), SN.data.profile.classResources.height - 6)
			self[1]:SetMinMaxValues(0, maxPower)
			self[1]:SetValue(power)
		end
	else
		if self:IsShown() then 
			self:Hide()
		end
	end
end

function SHARD:Lock()
	F.classpower.shards:Show()
end

function SHARD:Unlock()
	F.classpower.shards:Hide()
end

function SHARD:Reset()
end

function SHARD:ApplySettings()
	local spec = GetSpecialization()
	local number = F.classpower.shards.number

	F.classpower.shards:SetSize(F.classpower:GetWidth(), F.classpower:GetHeight())

	if spec == 1 or 3 then
		for i = 1, number do
			F.classpower.shards[i]:SetSize((F.classpower.shards:GetWidth() - (number + 1) * 3) / number, F.classpower.shards:GetHeight() - 4)
			F.classpower.shards[i]:SetStatusBarTexture(LSM:Fetch("statusbar", SN.data.profile.classResources.texture))
		end
	elseif spec == 2 then
		F.classpower.shards[1]:SetSize((F.classpower.shards:GetWidth() - 4), F.classpower.shards:GetHeight() - 4)
		F.classpower.shards[1]:SetStatusBarTexture(LSM:Fetch("statusbar", SN.data.profile.classResources.texture))
	end
end